
package corejini.chapter10;

import net.jini.discovery.*;
import net.jini.core.lookup.*;
import net.jini.core.entry.*;
import net.jini.core.event.*;
import java.util.Hashtable;
import java.io.IOException;
import java.io.Serializable;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
    
class DemoServiceProxy implements Serializable {
    public DemoServiceProxy() {
    }
}

public class Demo extends UnicastRemoteObject implements DiscoveryListener, Runnable, RemoteEventListener {
    protected final int LEASE_TIME = 10 * 1000; // 1 minute
    protected Hashtable registrations = new Hashtable();
    protected ServiceItem item;
    
    protected LeaseRenewalService leaseMgr = null;

    public Demo() throws IOException, RemoteException {
        super();
        Entry[] attrs = new Entry[1];
        attrs[0] = new net.jini.lookup.entry.Name("Lease Demo");
        item = new ServiceItem(null, new DemoServiceProxy(), null);
        
        LookupDiscovery disco = 
            new LookupDiscovery(LookupDiscovery.ALL_GROUPS);
        disco.addDiscoveryListener(this);
    }
    
    // Implement DiscoveryListener
    public void discovered(DiscoveryEvent ev) {
        System.out.println("discovered a lookup service!");
        ServiceRegistrar[] newregs = ev.getRegistrars();
        for (int i=0 ; i<newregs.length ; i++) {
            if (!registrations.containsKey(newregs[i])) {
                if (leaseMgr == null) {
                    findLeaseMgr(newregs[i]);
                }
                if (leaseMgr != null) {
                    registerWithLookup(newregs[i]);
                }
            }
                
        }
    }
    public void discarded(DiscoveryEvent ev) {
        ServiceRegistrar[] deadregs = ev.getRegistrars();
        for (int i=0 ; i<deadregs.length ; i++) {
            registrations.remove(deadregs[i]);
        }
    }
    
    protected void findLeaseMgr(ServiceRegistrar reg) {
        Class[] types = { LeaseRenewalService.class };
        ServiceTemplate tmpl = new ServiceTemplate(null, types, null);

        try {
            leaseMgr = (LeaseRenewalService) reg.lookup(tmpl);
        } catch (Exception ex) {
            System.err.println(ex.getMessage());
            ex.printStackTrace();
        }
    }
    
    protected synchronized void registerWithLookup(ServiceRegistrar registrar) {
        ServiceRegistration registration = null;

        try {
            registration = registrar.register(item, LEASE_TIME);
        } catch (RemoteException ex) {
            System.out.println("Couldn't register: " + ex.getMessage());
            return;
        }
        
        // If this is our first registration, use the service ID
        // returned to us.
        if (item.serviceID == null) {
            item.serviceID = registration.getServiceID();
            System.out.println("Set serviceID to " + item.serviceID);
        }
        
        registrations.put(registrar, registration);
        
        try {
            leaseMgr.renewFor(registration.getLease(), 1000 * 2 * 60, this);
        } catch (RemoteException ex) {
            System.err.println("Couldn't renew...");
            ex.printStackTrace();
        }
    }
    
    public void notify(RemoteEvent ev) throws RemoteException {
        System.err.println("GOT A REMOTE LEASE EVENT!  " + ev);
    }
    
    public void run() {
        while (true) {
            try {
                Thread.sleep(1000000);
            } catch (InterruptedException ex) {
            }
        }
    }
    public static void main(String args[]) {
        try {
            Demo hws = new Demo();
            new Thread(hws).start();
        } catch (Exception ex) {
            System.out.println("Couldn't create service: " +
                               ex.getMessage());
        }
    }
}
