
package corejini.chapter8;

import net.jini.discovery.LookupDiscovery;
import net.jini.discovery.DiscoveryEvent;
import net.jini.discovery.DiscoveryListener;
import net.jini.core.lookup.ServiceMatches;
import net.jini.core.lookup.ServiceItem;
import net.jini.core.lookup.ServiceTemplate;
import net.jini.core.lookup.ServiceRegistrar;
import net.jini.lookup.entry.ServiceInfo;
import net.jini.lookup.entry.Name;
import net.jini.core.entry.Entry;
import java.util.Hashtable;
import java.rmi.RemoteException;
import java.rmi.RMISecurityManager;
import java.io.IOException;

public class AllSearcher implements Runnable {
    protected Hashtable registrars = new Hashtable();
    protected Hashtable services = new Hashtable();
    protected ServiceTemplate tmpl;
    
    class Discoverer implements DiscoveryListener {
        public void discovered(DiscoveryEvent ev) {
            ServiceRegistrar[] newregs = ev.getRegistrars();
            for (int i=0 ; i<newregs.length ; i++) {
                addRegistrar(newregs[i]);
            }
        }
        public void discarded(DiscoveryEvent ev) {
            ServiceRegistrar[] newregs = ev.getRegistrars();
            for (int i=0 ; i<newregs.length ; i++) {
                removeRegistrar(newregs[i]);
            }
        }
    }
    
    public AllSearcher() throws IOException {
	if (System.getSecurityManager() == null) {
	    System.setSecurityManager(
		    new RMISecurityManager());
	}

        // build our template
    //    Entry[] attrTemplates = new Entry[1];
    //    attrTemplates[0] = new ServiceInfo(null, null, null, 
    //                                       null, null, null);
    //    tmpl = new ServiceTemplate(null, null, attrTemplates);
        tmpl = new ServiceTemplate(null, null, null);
        
        // set up for discovery
        LookupDiscovery disco = 
            new LookupDiscovery(LookupDiscovery.ALL_GROUPS);
        disco.addDiscoveryListener(new Discoverer());
    }
    
    protected synchronized void addRegistrar(ServiceRegistrar reg) {
        if (registrars.contains(reg.getServiceID()))
            return;
            
        registrars.put(reg.getServiceID(), reg);
        findServices(reg);
    }
    protected synchronized void removeRegistrar(ServiceRegistrar reg) {
        if (!registrars.contains(reg.getServiceID()))
            return;
        
        registrars.remove(reg.getServiceID());
    }
    
    void findServices(ServiceRegistrar reg) {
        try {
            ServiceMatches matches = reg.lookup(tmpl, Integer.MAX_VALUE);
        
            for (int i=0 ; i<matches.totalMatches ; i++) {
                if (services.contains(matches.items[i].serviceID))
                    continue;
            
        
                addService(matches.items[i]);
            }
        } catch (RemoteException ex) {
            System.err.println("Couldn't search for services: " +
                               ex.getMessage());
        }
    }
    
    protected void addService(ServiceItem item) {
        services.put(item.serviceID, item);
        System.out.println("New service found: " + item.serviceID);
        printServiceInfo(item);
    }
    
    protected void printServiceInfo(ServiceItem item) {
	System.out.println("Service " + item.serviceID + ": " + item.service);
        for (int i=0 ; i<item.attributeSets.length ; i++) {
	    Entry attr = item.attributeSets[i];
            if (attr instanceof ServiceInfo) {
                ServiceInfo info = (ServiceInfo) attr;
                System.out.println("    Name = " + info.name);
                System.out.println("    Manufacturer = " + info.manufacturer);
                System.out.println("    Vendor = " + info.vendor);
                System.out.println("    Version = " + info.version);
                System.out.println("    Model = " + info.model);
                System.out.println("    Serial Number = " + info.serialNumber);

		System.out.println("  PROXY is " + item.service);
            } else if (attr instanceof Name) {
                Name name = (Name) attr;
		System.out.println("    Name = " + name.name);
	    } else {
		System.out.println("    Attr = " + attr);
	    }
        }
    }
    
    public void run() {
        while (true) {
            try {
                Thread.sleep(Long.MAX_VALUE);
            } catch (InterruptedException ex) {
            }
        }
    }
    
    public static void main(String args[]) {
        try {
            AllSearcher searcher = new AllSearcher();
            new Thread(searcher).start();
        } catch (Exception ex) {
            System.err.println("Error starting service info searcher: " +
                               ex.getMessage());
            ex.printStackTrace();
        }
    }
}
