
package corejini.chapter8;

import net.jini.core.discovery.LookupLocator;
import net.jini.core.lookup.ServiceMatches;
import net.jini.core.lookup.ServiceItem;
import net.jini.core.lookup.ServiceTemplate;
import net.jini.core.lookup.ServiceRegistrar;
import net.jini.lookup.entry.ServiceInfo;
import net.jini.lookup.entry.Name;
import net.jini.core.entry.Entry;
import java.util.Hashtable;
import java.rmi.RemoteException;
import java.rmi.RMISecurityManager;
import java.io.IOException;

public class DirectSearcher implements Runnable {
    protected ServiceTemplate tmpl;
    
    public DirectSearcher(String lu) throws IOException, ClassNotFoundException {
	if (System.getSecurityManager() == null) {
	    System.setSecurityManager(
		    new RMISecurityManager());
	}

        tmpl = new ServiceTemplate(null, null, null);
        
	LookupLocator loc = new LookupLocator(lu);
        findServices(loc.getRegistrar());
    }
    
    void findServices(ServiceRegistrar reg) {
        try {
            ServiceMatches matches = reg.lookup(tmpl, Integer.MAX_VALUE);
        
            for (int i=0 ; i<matches.totalMatches ; i++) {
		printServiceInfo(matches.items[i]);
            }
        } catch (RemoteException ex) {
            System.err.println("Couldn't search for services: " +
                               ex.getMessage());
        }
    }
    
    protected void printServiceInfo(ServiceItem item) {
	System.out.println("++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++");
	System.out.println("Service " + item.serviceID + ": " + item.service);
        for (int i=0 ; i<item.attributeSets.length ; i++) {
	    Entry attr = item.attributeSets[i];
            if (attr instanceof ServiceInfo) {
                ServiceInfo info = (ServiceInfo) attr;
                System.out.println("    Name = " + info.name);
                System.out.println("    Manufacturer = " + info.manufacturer);
                System.out.println("    Vendor = " + info.vendor);
                System.out.println("    Version = " + info.version);
                System.out.println("    Model = " + info.model);
                System.out.println("    Serial Number = " + info.serialNumber);
            } else if (attr instanceof Name) {
                Name name = (Name) attr;
		System.out.println("    Name = " + name.name);
	    } else {
		// System.out.println("    Attr = " + attr);
	    }
        }
    }
    
    public void run() {
        while (true) {
            try {
                Thread.sleep(Long.MAX_VALUE);
            } catch (InterruptedException ex) {
            }
        }
    }
    
    public static void main(String args[]) {
	if (args.length != 1) {
	    System.err.println("Usage: DirectSearcher <url>");
	    System.exit(1);
	}
        try {
            DirectSearcher searcher = new DirectSearcher(args[0]);
            new Thread(searcher).start();
        } catch (Exception ex) {
            System.err.println("Error starting service info searcher: " +
                               ex.getMessage());
            ex.printStackTrace();
        }
    }
}
